open Base
open Exceptions

(** Signature of module supporting conversion from Sqlite raw values to a
    particular type *)
module type Sig = sig
  type t

  (** Converts Sqlite row of (header,data) into [t] or throws
      [Conversion_failed] exception if conversion is not possible *)
  val of_sql_row: (string * Sqlite3.Data.t) array -> t
end

(** Identity converter. Returns raw Sqlite data unchanged. Never fails. *)
module Id = struct
  type t = (string * Sqlite3.Data.t) array
  let of_sql_row = Fn.id
end

(** Converts single column INT result to [int64] *)
module Int64 = struct
  type t = int64
  let of_sql_row = function[@warning "-4"]
    | [| _, Sqlite3.Data.INT x |] -> x
    | data -> raise (Conversion_failed data)
end

(** Converts single column FLOAT result to [float] *)
module Float = struct
  type t = float
  let of_sql_row = function[@warning "-4"]
    | [| _, Sqlite3.Data.FLOAT x |] -> x
    | data -> raise (Conversion_failed data)
end

(** Convers single column BLOB or TEXT result to [string] *)
module String = struct
  type t = string
  let of_sql_row = function[@warning "-4"]
    | [| _, Sqlite3.Data.TEXT s |] -> s
    | [| _, Sqlite3.Data.BLOB s |] -> s
    | data -> raise (Conversion_failed data)
end

